/*
    Copyright (C) 2004, 2020 Hubert Vansteenhuyse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "subnet.h"
char out[81];
int arg_help = 0, arg_stdout = 0, arg_usenet = 0, arg_usehost = 0;
unsigned long ip = 0, mask = 0, wild = 0;

char *command_help[] = {
    "subnet [options] [IP] [SUBNETMASK]",
    "Options:",
    "  --stdout...........Runs in non interactive modus",
    "  --hosts [amount]...Subnet listing for specified amount of hosts",
    "  --nets [amount]....Subnet listing for specified amount of networks",
    "  --version..........Shows the version",
    "  --help.............Shows this help message",
    "",
    "  The options --ip, --hosts and --nets cannot be used together.",
    "",
    NULL
};

void yield(void){
#ifdef __WINDOWS__
    MSG msg;
    if(PeekMessage(&msg, (HWND)0, 0, 0, PM_REMOVE)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
#endif				/*#__WINDOWS__ */
}


void writefile(FILE * stream){
    int sw_histogram = (stream == stdout ? 0 : 1), c = 0;
    unsigned long net, cast, count = 1, totalnets = getnets();

    fprintf(stream, "\nSubnet Calculator %d.%d\n", VERSION_HI, VERSION_LO);
    for (c = 0; c < 80; c++)
        fprintf(stream, "-");
    fprintf(stream, "\n");
    fprintf(stream, "cidr: %s / %d \n", longtoip(ip), bits_mask(mask));
    fprintf(stream, "netmask: %s\n", longtoip(mask));
    fprintf(stream, "inverse netmask: %s\n", longtoip_inverse(mask));
    fprintf(stream, "class %c [%s]\n", getipclass(ip), iptype(ip));
    fprintf(stream, "#networks: %u, ", (unsigned) getnets());
    fprintf(stream, "#hosts: %u\n", (unsigned) gethosts());
    fprintf(stream, "magic number: %d, ", magicnumber());
    fprintf(stream, "#subnetbits: %d, %s\n", bits_subnet(), getipnetbits());
    for (c = 0; c < 80; c++)
        fprintf(stream, "-");
    fprintf(stream, "\n");
    fprintf(stream,
            "#        | NET             | BCAST           | RANGE                            \n");
    for (c = 0; c < 80; c++)fprintf(stream, "-");
    fprintf(stream, "\n");
    while (count <= totalnets) {
        if (sw_exit) return;

        net = ipcalcnet(count);
        cast = ipcalcbcast(count);
        memset(out, 0, 81);

        sprintf(out, "%u", (unsigned) count);
        while (strlen(out) < 9) sprintf(out + strlen(out), " ");
        sprintf(out + strlen(out), "| ");

        sprintf(out + strlen(out), "%s", longtoip(net));
        while (strlen(out) < 27) sprintf(out + strlen(out), " ");
        sprintf(out + strlen(out), "| ");
        sprintf(out + strlen(out), "%s", longtoip(cast));
        while (strlen(out) < 45) sprintf(out + strlen(out), " ");
        sprintf(out + strlen(out), "| ");
        sprintf(out + strlen(out), "%s - ", longtoip(net + 1));
        sprintf(out + strlen(out), "%s", longtoip(cast - 1));
        fprintf(stream, "%s\n", out);

        if (sw_histogram) histogram(count, totalnets);
        count++;
    }
}

//show help & exit
void helpermsg(void){
    int h = 0;
    while (command_help[h] != NULL) {
        fprintf(stdout, "%s\n", command_help[h]);
        h++;
    }
}

char	err_stderr_msg[1024 + 1],
        *err_stderr = err_stderr_msg;

int main(int argc, char *argv[]){
    int arg = 1;
    err_stderr[0] = 0;

    while (arg < argc && !*err_stderr) {

        if (strcmp(argv[arg], "--help") == 0) {
            arg_help = 1;
            arg++;
            continue;
        } else if (strcmp(argv[arg], "--stdout") == 0) {
            arg_stdout = 1;
            arg++;
            continue;
        } else if (strcmp(argv[arg], "--version") == 0) {
            fprintf(stdout, "subnet calculator version %d.%d\n", VERSION_HI, VERSION_LO);
            exit(0);
        } else if (strcmp(argv[arg], "--nets") == 0) {
            arg++;
            if (arg >= argc) {
                snprintf(err_stderr, 1024,	"in parameter --nets the amount of networks is missing");
                arg_help = 1;
            } else {
                arg_usenet = (long) atoi(argv[arg]);
                arg_usehost = 0;
            }
            arg++;
            continue;
        } else if (strcmp(argv[arg], "--hosts") == 0) {
            arg++;
            if (arg >= argc) {
                snprintf(err_stderr, 1024, "in parameter --hosts the amount of hosts is missing");
                arg_help = 1;
            } else {
                arg_usehost = (long) atoi(argv[arg]);
                arg_usenet = 0;
            }
            arg++;
            continue;
        } else if(!ip) {
            if(!isvalidip(argv[arg])) {
                snprintf(err_stderr, 1024, "Invalid IP address %s", argv[arg]);
                arg_help = 1;
            } else {
                ip = iptolong(argv[arg]);
            }
            arg++;
            continue;
        }
        if (!mask) {
            if (!isvalidmask(iptolong(argv[arg]))) {
                snprintf(err_stderr, 1024, "Invalid subnetmask %s", argv[arg]);
                arg_help = 1;
            } else {
                mask = iptolong(argv[arg]);
            }
            arg++;
            continue;
        } else {
            snprintf(err_stderr, 1024, "Unknown command %s", argv[arg]);
            arg_help = 1;
            arg++;
            break;
        }
    }


    if (arg_usenet) useamount_net(arg_usenet);
    if (arg_usehost) useamount_host(arg_usehost);

    if(!ip && !mask) {
        local_ip *local = local_ip_append(0);

        if(!get_local_ip(local)) {
            sprintf(local->address, "127.0.0.1");
            sprintf(local->mask, "255.0.0.0");
        }
        ip = iptolong(local->address);
        mask = iptolong(local->mask);

        local = local_ip_free(local);
    }

    wild = mask_inverse(mask);

    if (!arg_stdout) {
        //Run interactively
        if (initScreen()) {

            if(arg_help || *err_stderr)
                THESCREEN = 4;

            updateScreen();
            input();
            exitScreen();

        } else {
            fprintf(stderr, "Cannot initialize ncurses...\n");
        }
    } else {

        if(arg_help || *err_stderr) {
            if(*err_stderr) fprintf(stderr, "%s\n\n", err_stderr);
            helpermsg();
            return 0;
        }

        //Process all the output to stdout...
        writefile(stdout);
    }

    return EXIT_SUCCESS;
}
